

set -ex



pip check
jupyter lab --version
jlpm --version
jlpm versions
jupyter labextension list
jupyter lab path
jupyter server extension list
jupyter serverextension list
jupyter lab build
jupyter lab clean
exit 0
